import os,sys
from pprint import pprint
import shutil,glob,re
from datoeenos import tools,conf,commands,security,pkgconvert
import dns.zone
from dns.exception import DNSException
from dns.rdatatype import RdataType
from datetime import datetime
import ipaddress
import ssl
import warnings
warnings.filterwarnings('ignore', message='.*cryptography', )
import OpenSSL
import re
import socket
import urllib.parse
import tarfile

def setup_work_dir(workdir,log):
    result=False    
    if not os.path.isdir(workdir):
        try:
            os.makedirs(workdir)
            result=True
        except Exception as e:
            log(str(e))
            result=False
            pass
    else:
        try:
            shutil.rmtree(workdir)
            os.makedirs(workdir)
            result=True
        except Exception as e:
            log(str(e))
            result=False
            pass        
    return result

def get_shadow(backup,workdir,log):
    cp_shadow=backup+'/backup/.shadow'
    e_shadow=workdir+'/shadow'
    if os.path.isfile(cp_shadow):
        try:
            log("Coping Directadmin account password .. ")
            shutil.copy(cp_shadow,e_shadow)
            security.safe_chmod(e_shadow,0o600)
        except Exception as e:
            log(str(e))
            pass

def get_ips(da_data,log):
    log("Collecting IP Address(s) details")
    ipdata={}
    ipdata['ipv4']='127.0.0.1'
    ipdata['ipv6']='0000:0000:0000:0000:0000:0000:0000:0001'
    ipdata['status']='off'
    ipdata['status6']='off'
    if 'serverip' in da_data and bool(da_data['serverip']):
        ipdata['ipv4']=da_data['serverip']
        log("IPv4 found on account : %s" %da_data['serverip'])        
        if str(da_data['ip']).__contains__(".") and  da_data['serverip'] != da_data['ip']:
            log("Found dedicated IPv4 for the account")
            ipdata['status']='on'
    if 'serveripv6' in da_data and bool(da_data['serveripv6']):
        ipdata['ipv6']=da_data['serveripv6']
        log("IPv6 found on account : %s" %da_data['serveripv6'])
        if str(da_data['ip']).__contains__(":") and  da_data['serveripv6'] != da_data['ip']:
            log("Found dedicated IPv6 for the account")
            ipdata['status6']='on'
    return ipdata
def get_nameserver(da_data,domain):
    nss=list()
    for ns in ['ns1','ns2','ns3','ns4']:
        if ns in da_data and bool(da_data[ns]):
            nss.append(da_data[ns])
    if not bool(nss):
        nss.append("dns1."+domain)
        nss.append("dns2."+domain)
    # pprint(nss)
    return nss

def is_reseller(user,backup,log):
    status=False
    rfile=backup+'/backup/user.conf'
    if os.path.isfile(rfile) and os.stat(rfile).st_size >0:
        rdata=tools.readconfig(rfile,log)    
        if 'usertype' in rdata and bool(rdata['usertype']):
            if rdata['usertype'] in ['admin','reseller']:               
                status=True
            else:
                status=False
    return status

def is_reseller_self_own(user,backup,log):
    status=False
    rfile=backup+'/backup/users.list'
    if os.path.isfile(rfile) and os.stat(rfile).st_size >0:
        rdata=tools.readplain(rfile,log)
        if user in rdata:
            status=True
    return status

def convert_cp_userdata(user,backup,data,log):
    udata=dict()
    ufile=backup+'/backup/user.conf'    
    pdata=tools.readconfig(ufile,log)    
    isconverted=False
    if bool(pdata):
        for item in pdata:
            if pdata[item]=="ON":
                pdata[item]="on"
            if pdata[item]=="OFF":
                pdata[item]="off"
            if item=='ssh':
                if pdata[item]=="off":
                    pdata[item]="no"
                else:
                    pdata[item]="yes"        
        if bool(pdata):
            # pprint(pdata)
            udata=conf.user_config(user) 
            if 'domain' in pdata and bool(pdata['domain']):
                udata['domain']=pdata['domain']
            else:
                udata['domain']='changethisdomainname.com'                            
            plan_maps=conf.user_map()
            for item in pdata:
                try:
                    udata[plan_maps[item]]=pdata[item]
                except Exception:
                    pass    
                udata['feature']='Default'           
            date_string=udata['setuptime']
            try:
                isostring=datetime.strptime(date_string,'%a %b %d %H:%M:%S %Y')
            except:
                isostring=datetime.now().strftime("%Y-%m-%d %H:%M:%S")               
                pass
            udata['setuptime']=str(isostring)           
            main_domain=udata['domain']
            log("Finding main domain : %s" %main_domain)
            udata['dedicatedip']=get_ips(pdata,log)            
            # udata['homedir']=get_homedir(user,backup,main_domain,log)
            # log("Finding HomeDir : %s" %udata['homedir'])
            udata['nameservers']=get_nameserver(pdata,main_domain)            
            log("Collecting name servers : %s" %','.join(udata['nameservers']))
            if udata['package']== 'admin':
                udata['package']='Default'
            # pprint(udata)
            # sys.exit(0)
            log("Checking if it is reseller account ..")
            isreseller=is_reseller(user,backup,log)
            selfown="off"
            if isreseller:
                udata['reseller']['status']="on"
                log("This account is reseller")
                log("Checking for self own account .. ")
                isselfown=is_reseller_self_own(user,backup,log)
                if isselfown:
                    udata['owner']=user
                    log("This user own his own account")
                    selfown="on"
                else:                    
                    log("This user doesn't own his account")
                    selfown="off"
                udata['reseller']['selfown']=selfown
            else:
                udata['reseller']['status']="off"
                udata['reseller']['selfown']="off"
                log("This is not a reseller account ..")
            if bool(data['skipresellerconfig']):
                log("You chose to skip reseller privileges ,, so disabing reseller privileges")
                udata['reseller']['status']="off"
                udata['reseller']['selfown']="off"            
            if  not bool(udata['contactemail']):                
                nco=udata['user']+'@'+udata['domain']
                log("Directadmin account doesn't have a contact email, settings a default one , %s" %nco)
                udata['contactemail']=nco            
            if pdata['ssh']== 'yes':
                udata['shell']['shell']='/usr/local/eenos/bin/jailshell'
                udata['shell']['status']='on'
            isconverted=True
        else:
           udata=conf.user_config(user) 
    else:
        udata=conf.user_config(user)    
    if isconverted:
        return udata
    else:
        return dict()
    
def get_user_config(backup,workdir,user,data,log):
    ucfdir=workdir+'/userconfig'
    userdata=convert_cp_userdata(user,backup,data,log)
    uconfig=ucfdir+'/'+user
    try:
        os.makedirs(ucfdir)        
    except Exception as e:
        log(str(e))
        pass
    try:
        tools.writeyaml(uconfig,userdata,log)        
        log("Converting Eenos user configuration : userconfig/%s" %user)
    except Exception as e:
        log(str(e))
        pass


def get_reseller_conf(userdata,backup,workdir,user,log):    
    rdata=conf.default_reseller_conf(reseller=user)
    rcdir=workdir+'/resellerconfig'
    rlimitfile=backup+'/backup/reseller.conf'    
    if os.path.isfile(rlimitfile) and os.stat(rlimitfile).st_size >0:
        rlimit=tools.readconfig(rlimitfile,log)          
    else:
        rlimit=dict()
    
    if bool(rlimit):
        if 'nusers' in rlimit and bool(rlimit['nusers']):
            reseller_limt=rlimit['nusers']
        else:
            reseller_limt='unlimited'
        if 'bandwidth' in rlimit and bool(rlimit['bandwidth']):
            reseller_bandwidth=rlimit['bandwidth']
        else:
            reseller_bandwidth='unlimited'
        if 'quota' in rlimit and bool(rlimit['quota']):
            reseller_quota=rlimit['quota']
        else:
            reseller_quota='unlimited'
        rdata['reseller_limt'] = reseller_limt
        rdata['reseller_bandwidth'] = reseller_bandwidth
        rdata['reseller_quota'] = reseller_quota      
    
    nss=get_nameserver(rlimit,userdata['domain'])
    rdata['nameservers']=nss
    rdata['contactemail']=userdata['contactemail']
    rdata['ips']['ipv4']=userdata['dedicatedip']['ipv4']
    rdata['ips']['ipv6']=userdata['dedicatedip']['ipv6']
   
    acfile=backup+'/backup/users.list'
    accounts=list()
    if os.path.isfile(acfile) and os.stat(acfile).st_size > 0:
        acs=tools.readplain(acfile,log)
        if bool(acs):
            log("Collecting reseller sub account names .. ")
            accounts=acs   
    if userdata['user']==userdata['owner']:
        accounts.append(user)
    rdata['accounts']=accounts
    result=False
    try:
        if not os.path.isdir(rcdir):
            log("Creating reseller config dir")
            os.makedirs(rcdir)
    except Exception as e:
        log(str(e))
        pass
    rcofile=rcdir+'/'+user
    try:
        log("Creating reseller config  .. ")
        tools.writeyaml(rcofile,rdata,log)
        result=True
    except Exception as e:
        log(str(e))
        pass        
    return result

def get_reseller_packages(backup,workdir,user,log):
    log("Converting Reseller packages ")
    rcpkgdir=workdir+'/resellerpackages'
    try:
        if not os.path.isdir(rcpkgdir):
            os.makedirs(rcpkgdir)
    except Exception as e:
        log(str(e))
        pass
    cppkg_dir=backup+'/backup/packages'
    packages={}
    if os.path.isdir(cppkg_dir):
        pfiles=glob.glob(cppkg_dir+'/*.pkg')
        pfiles.sort()
        if bool(pfiles):
            for pfile in pfiles:
                if os.path.isfile(pfile):                     
                    pname=pfile.split('/')[-1]
                    plan_name=str(pname).split(".")[0]
                    log("Converting Directadmin Package : %s" %plan_name)
                    packages[plan_name]=pkgconvert.convert_cp(pfile,log,plan_name,owner=user)                
    if bool(packages):
        for pname in packages:
            pfile=rcpkgdir+'/'+pname+'.yaml'            
            log("[ %s ] Creating Eenos reseller package : %s" %(user,pname))
            tools.writeyaml(pfile,packages[pname],log)
        # pprint(packages)

def get_reseller_features(workdir,log):
    rcfdir=workdir+'/resellerfeatures'
    try:
        if not os.path.isdir(rcfdir):
            log("Creating reseller feature directory")
            os.makedirs(rcfdir)
    except Exception as e:
        log(str(e))
        pass

def collect_hostname():
    return  str(socket.gethostname())


def create_userdata_dirs(workdir,log):
    log("Creating Eenos userdata dirs ...")
    ucdir=workdir+'/userdata'
    if not os.path.isdir(ucdir):
        try:
            os.makedirs(ucdir)
        except Exception as e:
            log(str(e))
            pass
    thisdir=ucdir+'/mysql'
    if not os.path.isdir(thisdir):
        try:
            log("Creating mysql dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass
    thisdir=ucdir+'/vhosts/httpd'
    if not os.path.isdir(thisdir):
        try:
            log("Creating httpd dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass 
    thisdir=ucdir+'/vhosts/nginx'
    if not os.path.isdir(thisdir):
        try:
            log("Creating nginx dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass   
    thisdir=ucdir+'/pgsql'
    if not os.path.isdir(thisdir):
        try:
            log("Creating pgsql dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass 
    thisdir=ucdir+'/dns'
    if not os.path.isdir(thisdir):
        try:
            log("Creating dns dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass 
    thisdir=ucdir+'/domains'
    if not os.path.isdir(thisdir):
        try:
            log("Creating domains dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass  
    thisdir=ucdir+'/ssl'
    if not os.path.isdir(thisdir):
        try:
            log("Creating ssl dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass  
    thisdir=ucdir+'/dovecot'
    if not os.path.isdir(thisdir):
        try:
            log("Creating dovecot dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass
    # Mail folders 
    thisdir=ucdir+'/mail'
    if not os.path.isdir(thisdir):
        try:
            log("Creating mail dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass 
    # Mail folders 
    thisdir=ucdir+'/mail/domainfilter'
    if not os.path.isdir(thisdir):
        try:
            log("Creating Exim Domainfilter dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass   
    thisdir=ucdir+'/mail/mailman'
    if not os.path.isdir(thisdir):
        try:
            log("Creating Mailing list dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass  
    thisdir=ucdir+'/mail/accountfilter'
    if not os.path.isdir(thisdir):
        try:
            log("Creating Exim Account filter dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass  
    thisdir=ucdir+'/mail/forwarder'
    if not os.path.isdir(thisdir):
        try:
            log("Creating Exim Email Forwarder dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass  
    thisdir=ucdir+'/mail/distribute'
    if not os.path.isdir(thisdir):
        try:
            log("Creating Exim Distribution List dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass         
    thisdir=ucdir+'/mail/pipe'
    if not os.path.isdir(thisdir):
        try:
            log("Creating Exim Email piping dir")
            os.makedirs(thisdir)
        except Exception as e:
            log(str(e))
            pass 

def build_domain_data(domain,docroot,user,udata,backup,log,cdomain=None):
    log("Collecting domain configuration for : %s" %domain)
    serveradmin='webmster@'+domain   
    ipv4=udata['dedicatedip']['ipv4']
    log( "[ %s ] - IPv4  - %s" %(domain,ipv4))
    if 'ipv6' in udata['dedicatedip'] and bool(udata['dedicatedip']['ipv6']):
        ipv6=udata['dedicatedip']['ipv6']
        log( "[ %s ] - IPv6  - %s" %(domain,ipv6))
    else:
        ipv6=None   
    serveralias=list()
    serveralias.append('www.'+domain)   
    log( "[ %s ] - Server Aliases  - %s" %(domain,','.join(serveralias)))
    domdata=conf.domain_conf(domain,serveradmin,user,docroot,ipv4,ipv6,alias=serveralias)    
    return domdata

def  extract_cp_zone(zonefile,domain,log):
    log("Collecting dns from dierctadmin zone : %s" %domain)
    # log(zonefile)
    try:
        z=dns.zone.from_file(zonefile,domain)
    except Exception as e:
        log(str(e))
        z=None
        pass
    data=list()
    get_rid_of_cpanel=['whm','cpanel','webdisk','webmail','default._domainkey','cpcontacts','cpcalendars','_cpanel-dcv-test-record']
    if z:
        for (name, rdataset) in z.iterate_rdatasets():
            # print(rdataset.rdtype)
            # print(rdataset.rdtype.name)
            #     continue
            try:
                zname=str(name)
                zdata=str(rdataset)
                xdata={}
                if rdataset.rdtype == RdataType.SOA:            
                    continue
                elif rdataset.rdtype == RdataType.NS :
                    rans=list(filter(None,str(rdataset).split("\n")))
                    for ns in rans:
                        xdata={}
                        xdata['@']=ns
                        data.append(xdata)            
                    # # pprint(dir(rdataset))
                    # print(str(rdataset).split("\n"))
                    # print("-=========")
                elif rdataset.rdtype == RdataType.MX :
                    rans=list(filter(None,str(rdataset).split("\n")))                                      
                    for mx in rans:
                        xdata={}
                        if  str(mx).endswith('@'):
                            xdata['@']=mx
                        elif not str(mx).endswith('.'):
                            mx=mx+'.'+domain+'.' 
                            xdata['@']=mx
                        else:
                            xdata['@']=mx                        
                        data.append(xdata) 
                elif rdataset.rdtype == RdataType.TXT and zname =='@':
                    rans=list(filter(None,str(rdataset).split("\n")))                    
                    for tx in rans:
                        xdata={}                        
                        xdata['@']=tx
                        data.append(xdata)
                elif not zname.startswith(tuple(get_rid_of_cpanel)):
                    # print("%s - %s " %(zname,zdata)) 
                    xdata={}
                    xdata[zname]= zdata 
                    data.append(xdata)
            except Exception as e:
                log(str(e))  
                pass
    if bool(data):
        return data
    else:
        return list()

def create_dns_file(domain,workdir,domain_data,userdata,backup,log):
    zonefile=workdir+'/userdata/dns/'+domain
    cpzonefile=backup+'/backup/'+domain+'/'+domain+'.db'
    if os.path.isfile(cpzonefile) and os.stat(cpzonefile).st_size > 0:
        cp_records=extract_cp_zone(cpzonefile,domain,log)
        # pprint(cp_records)
    else:
        cp_records=list()
    if 'ips' in domain_data and 'ipv4' in domain_data['ips'] and bool(domain_data['ips']['ipv4']):
        ipv4=domain_data['ips']['ipv4']
    else:
        ipv4=None
    if 'ips' in domain_data and 'ipv6' in domain_data['ips'] and bool(domain_data['ips']['ipv6']):
        ipv6=domain_data['ips']['ipv6']
    else:
        ipv6=None
    NS=userdata['nameservers']
    MX=dict()
    MX['0']=domain
    zone=conf.dns_conf(domain,ipv4,ipv6,NS,MX)
    nss=list()
    mxs={}
    arecords={}
    aaaarecords={}
    cnames={}
    gtxt=list()
    txts={}
    spfcustom=None
    if bool(cp_records):        
        for record in cp_records:
            rcdata=list(record.items())[0]
            rc_name=rcdata[0]
            rc_val=rcdata[-1]
            rc_type=rc_val.split(" ")
            record_type=rc_type[2]
            if record_type == 'NS': 
                nsname=str(rc_type[3])
                if nsname.endswith('.'):
                   nsname=nsname[:-1]      
                nss.append(nsname)
            if record_type == 'MX':
                mxtarget=rc_type[4]
                if mxtarget=='@':
                   mxtarget=domain
                elif  mxtarget.endswith('.'):
                    mxtarget=mxtarget[:-1]                
                mxs[rc_type[3]]=mxtarget
            if record_type == 'A':
                if rc_name == '@':
                    zone['ips']['ipv4']=rc_type[3]
                    zone['spf']['ipv4'].append(rc_type[3])                    
                else:
                    arecords[rc_name]={}
                    arecords[rc_name]['ip']=rc_type[3]
                    arecords[rc_name]['ttl']=rc_type[0]
            if record_type == 'AAAA':
                if rc_name == '@':
                    zone['ips']['ipv6']=ipaddress.ip_address(rc_type[3]).exploded
                    zone['spf']['ipv6'].append(ipaddress.ip_address(rc_type[3]).exploded)
                else:
                    aaaarecords[rc_name]={}
                    aaaarecords[rc_name]['ip']=ipaddress.ip_address(rc_type[3]).exploded  
                    aaaarecords[rc_name]['ttl']=rc_type[0]
            if record_type == 'CNAME':
                cntarget=rc_type[3]
                if rc_name != '@' and cntarget != '@':                    
                    if cntarget.endswith('.'):
                        cntarget=cntarget[:-1]
                    cnames[rc_name]={}
                    cnames[rc_name]['domain']=cntarget
                    cnames[rc_name]['ttl']=rc_type[0]
            if record_type == 'TXT':
                tdata=rc_val.split(" ",3)
                if rc_name == '@':
                    if 'spf' in tdata[3]:
                        if str(tdata[3]).startswith('"') and str(tdata[3]).endswith('"'):
                            tdata[3]=tdata[3][1:-1]                        
                        spfcustom=tdata[3]
                    else:
                        if str(tdata[3]).startswith('"') and str(tdata[3]).endswith('"'):
                            tdata[3]=tdata[3][1:-1]
                        gtxt.append(tdata[3])
                else:
                    if str(tdata[3]).startswith('"') and str(tdata[3]).endswith('"'):
                        tdata[3]=tdata[3][1:-1]
                    txts[rc_name]={}
                    txts[rc_name]['value']=tdata[3]
                    txts[rc_name]['ttl']=tdata[0]
                # pprint(tdata)
        try:
            # Remove duplicates
            zone['spf']['ipv4']=list(set(zone['spf']['ipv4'])) 
            zone['spf']['ipv6']=list(set(zone['spf']['ipv6'])) 
        except Exception as e:
            log(str(e))
            pass            
    if bool(nss):        
        zone['NS']=nss
    if bool(mxs):
        zone['MX']=mxs        
    if bool(arecords):
        zone['A']=arecords
    if bool(aaaarecords):
        zone['AAAA']=aaaarecords
    if bool(cnames):
        zone['CNAME']=cnames
    if bool(gtxt):
        zone['GTXT']=gtxt
    if bool(txts):
        zone['TXT']=txts
    if bool(spfcustom):
        zone['spfcustom']=spfcustom        
    if bool(zone):
        return zone
    else:
        return dict()

def get_subdomain_docroot(backup,domain,homedir,log):  
    sfile=backup+'/backup/'+domain+'/domain.subdomains.docroot.override'
    data={}
    epdata=list()
    if os.path.isfile(sfile) and os.stat(sfile).st_size >0:
        pdata=tools.readplain(sfile,log)
        if bool(pdata):
            for item in pdata:
                enc=urllib.parse.unquote(item)
                epdata.append(enc)
    if bool(epdata):
        for sub in epdata:
            name=sub.split("=")[0]
            public=sub.split("&")[-1]
            docroot=public.split("=")[-1]
            data[name]=homedir+docroot    
    return data

def update_subdomain_zone(subdomain,maindomain,name,sddon_data,workdir,backup,log):
    log("Adding subdomain to DNS Zone  subdomain : %s , main domain : %s , name : %s" %(subdomain,maindomain,name))
    zonefile=workdir+'/userdata/dns/'+maindomain
    # print(zonefile)
    # pprint(sddon_data)
    if os.path.isfile(zonefile) and os.stat(zonefile).st_size > 0:
        zdata=tools.readyaml(zonefile,log)
        # pprint(zdata)
        if name in zdata['A']:           
            del zdata['A'][name]
        if 'www.'+name in zdata['A']:        
            del zdata['A']['www.'+name]
        if name in zdata['AAAA']:
            del zdata['AAAA'][name]
        if 'www.'+name in zdata['AAAA']:
            del zdata['AAAA']['www.'+name]
        zdata['sub'][subdomain]={}
        zdata['sub'][subdomain]['name']=name
        zdata['sub'][subdomain]['ttl']='14400'
        if 'ips' in sddon_data and 'ipv4' in sddon_data['ips'] and bool(sddon_data['ips']['ipv4']):
            zdata['sub'][subdomain]['ipv4']=sddon_data['ips']['ipv4']
        if 'ips' in sddon_data and 'ipv6' in sddon_data['ips'] and bool(sddon_data['ips']['ipv6']):
            zdata['sub'][subdomain]['ipv6']=sddon_data['ips']['ipv6']
        tools.writeyaml(zonefile,zdata,log)
        # pprint(zdata)
    else:
        log("== Main domain zone data file not found , skippings dns of %s" %subdomain)

def collect_domains(userdata,backup,workdir,user,log):
    log("Collecting domain names")
    main={}
    main_domain=userdata['domain']
    main['main_domain']=main_domain
    main['addon_domains']=list()
    main['parked_domains']=list()
    main['sub_domains']={}
    alldomains=list()
    alldomains.append(main_domain)
    imaps=backup+'/imap'
    homedir=userdata['homedir']
    subdomains_docroot={}
    if os.path.isdir(imaps):
        ifiles=glob.glob(imaps+'/*')
        if bool(ifiles):
            for dom in ifiles:
                name=dom.split('/')[-1]
                if name == main_domain:
                    continue
                elif os.path.islink(dom):
                    main['parked_domains'].append(name)
                    alldomains.append(name)
                    log("Found parked domain- %s" ,name)
                elif os.path.isdir(dom):
                    main['addon_domains'].append(name)
                    alldomains.append(name)
                    log("Found addon domain- %s" ,name)
    for sd in  alldomains:
        sdfile=backup+'/backup/'+sd+'/subdomain.list'
        sd__docroot=get_subdomain_docroot(backup,sd,homedir,log)
        if os.path.isfile(sdfile) and os.stat(sdfile).st_size >0:
            log("[ %s ] Collecting subdomain "%sd)
            sdomains=tools.readplain(sdfile,log)
            if bool(sdomains):
                for sub in sdomains:
                    name=sub
                    of=sd
                    domain=sub+'.'+sd
                    log("[ %s ] subdomain found - %s" %(sd,domain))
                    main['sub_domains'][domain]={}
                    main['sub_domains'][domain]['name']=name
                    main['sub_domains'][domain]['of']=of
                    if sub in sd__docroot:
                        subdomains_docroot[domain]=sd__docroot[sub]
                    else:
                         subdomains_docroot[domain]=homedir+'/'+domain
    docroot=userdata['homedir']+'/public_html'    
    main_data=build_domain_data(main_domain,docroot,user,userdata,backup,log)
    dfile=workdir+'/userdata/domains/'+main['main_domain']
    try:
        log("Creating Main domain configuration : %s" %main['main_domain'])
        tools.writeyaml(dfile,main_data,log)
        zdata=create_dns_file(main_domain,workdir,main_data,userdata,backup,log)
        if bool(zdata):
            zfile=workdir+'/userdata/dns/'+main_domain
            log("Creating DNS zone data of %s" %main_domain)
            tools.writeyaml(zfile,zdata,log)
    except Exception as e:
        log(str(e))
        pass
    if bool(main['addon_domains']):
        cpaddons=main['addon_domains']
        # pprint(cpaddons)
        for addon in cpaddons:
            docroot=homedir+'/domains/'+addon+'/public_html'            
            addon_data=build_domain_data(addon,docroot,user,userdata,backup,log)
            domfile=workdir+'/userdata/domains/'+addon
            try:
                log("Creating Addon domain configuration : %s" %addon)
                tools.writeyaml(domfile,addon_data,log)
                zdata=create_dns_file(addon,workdir,addon_data,userdata,backup,log)
                if bool(zdata):
                    zfile=workdir+'/userdata/dns/'+addon
                    log("Creating DNS zone data of %s" %addon)
                    tools.writeyaml(zfile,zdata,log)
            except Exception as e:
                log(str(e))
                pass

    parkeddomain=main['parked_domains']    
    if bool(parkeddomain):
        dfile=workdir+'/userdata/domains/'+main_domain
        pdata=tools.readyaml(dfile,log)        
        for pdom in parkeddomain:
            pdom_data=pdata
            pdom_data['servername']=pdom
            pdom_data['serveralias']=list()
            pdom_data['serveralias'].append('www.'+pdom)
            pdom_data['serveradmin']='webmaster@'+pdom
            try:
                domfile=workdir+'/userdata/domains/'+pdom
                log("Creating Parked domain configuration : %s" %pdom)
                tools.writeyaml(domfile,pdom_data,log)                
                zdata=create_dns_file(pdom,workdir,pdom_data,userdata,backup,log)
                if bool(zdata):
                    zfile=workdir+'/userdata/dns/'+pdom
                    log("Creating DNS zone data of %s" %pdom)
                    tools.writeyaml(zfile,zdata,log)
            except Exception as e:
                log(str(e))
                pass

    subdomains=main['sub_domains']    
    if bool(subdomains):
        # pprint(subdomains)
        for subdomain in subdomains:
            dfile=backup+'/userdata/'+subdomain
            try:
                docroot=subdomains_docroot[subdomain]
            except:
                docroot=homedir+'/'+subdomain
                pass
            sddon_data=build_domain_data(subdomain,docroot,user,userdata,backup,log)
            domfile=workdir+'/userdata/domains/'+subdomain
            try:
                log("Creating Sub domain configuration : %s" %subdomain)
                tools.writeyaml(domfile,sddon_data,log)                
                update_subdomain_zone(subdomain,subdomains[subdomain]['of'],subdomains[subdomain]['name'],sddon_data,workdir,backup,log)               
            except Exception as e:
                log(str(e))
                pass
    mfile=workdir+'/userdata/main'
    try:
        log("Creating main data file ..")
        tools.writeyaml(mfile,main,log)
    except Exception as e:
        log(str(e))
        pass  

def colloect_ftp_accounts(backup,workdir,userdata,log):
    log("Collecting FTP Accounts ")
    cpftp=glob.glob(backup+'/backup/*/ftp.passwd')
    eftp=workdir+'/userdata/.ftp.passwd'
    accounts={}
    homedir=userdata['homedir']
    user=userdata['user']
    ugid=tools.getugid(user)
    if bool(cpftp):
        for ffile in cpftp:
            if os.path.isfile(ffile) and os.stat(ffile).st_size > 0:
                ssdata=tools.readplain(ffile,log)
                for data in ssdata:
                    thisdata=data.split("&")
                    userdata=thisdata[0].split("=")
                    if '@' in userdata[0]:
                        accounts[userdata[0]]={}
                        accounts[userdata[0]]['password']=userdata[-1]
                        accounts[userdata[0]]['uid']=ugid
                        accounts[userdata[0]]['gid']=ugid
                        mypath=thisdata[-2].split("=")[-1]                        
                        if str(mypath).endswith('/'):
                            mypath=mypath[:-1]
                        if str(mypath).startswith(homedir):
                            accounts[userdata[0]]['home']=mypath
                        else:
                            tp=str(mypath).split(user,1)[-1]                            
                            mypath=homedir+tp
                            accounts[userdata[0]]['home']=mypath                        
    else:
        log("Not ftp accounts found")
    if bool(accounts):
        data=list()
        for ac in accounts:
            log("[ FTP ] - %s" %ac)
            item=ac+':'+accounts[ac]['password']+':'+accounts[ac]['uid']+':'+accounts[ac]['gid']+':'+user+':'+accounts[ac]['home']+':/bin/ftpsh'
            data.append(item)
        if bool(data):
            try:
                tools.writeplain(eftp,data,log)
                security.safe_chmod(eftp,0o640)
            except Exception as e:
                log(str(e))
                pass
        else:
            log("Not ftp accounts found")
   

def collect_cron_jobs(backup,workdir,user,log):
    log("Collecting Directadmin cron jobs")
    cpcronf=backup+'/backup/crontab.conf'  
    cpcrons=list()
    crontab={}
    if os.path.isfile(cpcronf) and os.stat(cpcronf).st_size > 0:
        rdata=tools.readconfig(cpcronf,log)
        for item in rdata:
            xd=rdata[item].split(" ")
            if len(xd) >5:
                cpcrons.append(rdata[item])
        if bool(cpcrons):
            for cron in cpcrons:                                
                try:                        
                    cdata=cron.split(" ",5)
                    ccommand=str(cdata[5])                    
                    cschedule=' '.join(cdata[0:5:1])
                    cronid=tools.getid()                        
                    crontab[cronid]={}
                    crontab[cronid]['command']=ccommand
                    crontab[cronid]['schedule']=cschedule
                except Exception as e:
                    log(str(e))
                    pass        
        if bool(crontab):
            # pprint(crontab)
            ecron=workdir+'/userdata/crontab'            
            try:
                log("Creating Eenos cron tab  data file ..")
                tools.writeyaml(ecron,crontab,log)
            except Exception as e:
                log(str(e))
                pass            
        else:
            log("No cron jobs foud ..")
    else:
        log("No cron jobs foud ..")

def  collect_mail_filter_forwardor_pip(backup,workdir,data,userdata,log):
    homedir=userdata['homedir']
    user=userdata['user']
    ugid=tools.getugid(user)
    if bool(data['skipemail']):
        log("Skipping email accounts,forwarders, email pips and email filters")
    else:
        log("[ EMAIL ] Collecting Directadmin Email accounts")
        accountdir=workdir+'/homedir/etc'
        if not os.path.isdir(accountdir):
            try:
                os.makedirs(accountdir)
                security.safe_chmod(accountdir,0o750)
            except Exception as e:
                log(str(e))
                pass    
        pfiles=glob.glob(backup+'/backup/*/email/passwd')
        accounts=dict()
        if bool(pfiles):
            for pfile in pfiles:
                if os.path.isfile(pfile) and os.stat(pfile).st_size > 0:
                    qfile=str(pfile).replace("passwd","quota")
                    # print(pfile)
                    domain=pfile.split("/")[-3]
                    if not domain in accounts:
                        accounts[domain]=dict()                        
                    pdata=tools.readplain(pfile,log)
                    for item in pdata:
                        its=item.split(":")
                        euser=its[0]
                        epass=its[-1]
                        accounts[domain][euser]=dict()
                        accounts[domain][euser]['password']=epass
                        accounts[domain][euser]['home']=homedir+'/mail/'+domain+'/'+euser
                        accounts[domain][euser]['uid']=ugid
                        accounts[domain][euser]['gid']=ugid
                        accounts[domain][euser]['epoch']=str((datetime.utcnow() - datetime(1970,1,1)).days)
                    tdir=accountdir+'/'+domain
                    if not os.path.isdir(tdir):
                        try:
                            os.makedirs(tdir)                            
                        except Exception as e:
                            log(str(e))
                            pass
                    tdfile=tdir+'/quota'
                    if os.path.isfile(qfile):
                        try:
                            log("[ EMAIL ] Collecting email quota files on domain %s" %domain)
                            shutil.copy(qfile,tdfile)
                        except Exception as e:
                            log(str(e))
                            pass                    
        if bool(accounts):
            for domain in accounts:
                pwdlist=list()
                shwlist=list()
                pwfile=accountdir+'/'+domain+'/passwd'
                shfile=accountdir+'/'+domain+'/shadow'
                for name in accounts[domain]:
                    email=name+'@'+domain
                    log("[ EMAIl ] Found email account - %s" %email)
                    pwddata=name+':x:'+accounts[domain][name]['uid']+':'+accounts[domain][name]['gid']+'::'+accounts[domain][name]['home']+':'+homedir
                    pwdlist.append(pwddata)
                    shadata=name+':'+accounts[domain][name]['password']+':'+accounts[domain][name]['epoch']+'::::::'
                    shwlist.append(shadata)
                if bool(pwdlist):
                    try:
                        log("Creating Email password file for domain %s" %domain)
                        tools.writeplain(pwfile,pwdlist,log)
                        security.safe_chmod(pwfile,0o640)
                    except Exception as e:
                        log(str(e))
                        pass
                if bool(shwlist):
                    try:
                        log("Creating Email shadow file for domain %s" %domain)
                        tools.writeplain(shfile,shwlist,log)
                        security.safe_chmod(shfile,0o600)
                    except Exception as e:
                        log(str(e))
                        pass    
        # End of email account creation
        forworders={}
        mailpips={}
        alsfs=glob.glob(backup+'/backup/*/email/aliases')
        if bool(alsfs):
            for alias in alsfs:
                if os.path.isfile(alias) and os.stat(alias).st_size > 0:
                    # print(alias)
                    domain=alias.split("/")[-3]
                    adata=tools.readplain(alias,log)
                    for item in adata:
                        idata=item.split(":",1)
                        frm=idata[0]
                        to=idata[-1].strip()
                        if not str(frm).__contains__('@'):
                            frm=frm+'@'+domain
                        if str(to).__contains__("|"):
                            if not domain in mailpips:
                                mailpips[domain]=dict()
                            if to.startswith('"'):
                                to=to[1:]
                            if to.endswith('"'):
                                to=to[:-1]   
                            mailpips[domain][frm]=to
                            log("[ EMAIL ] Email pip found : %s" %frm)
                        else:
                            if not domain in forworders:
                                forworders[domain]=dict()
                            forworders[domain][frm]=to
                            log("[ EMAIL ] Email alias/forwarder found : %s" %frm)
            if bool(forworders):
                eenos_forward_dir=workdir+'/userdata/mail/forwarder'
                if not os.path.isdir(eenos_forward_dir):
                    try:
                        os.makedirs(eenos_forward_dir)
                    except:
                        pass
                for domain in forworders:
                    ffile=eenos_forward_dir+'/'+domain                    
                    try:
                        log("[ EMAIL ] Generating email alias and forwarders of %s" %domain)
                        tools.writeyaml(ffile,forworders[domain],log)
                    except Exception as e:
                        log(str(e))
                        pass        
                # pprint(forworders)
            if bool(mailpips):
                eenos_pip_dir=workdir+'/userdata/mail/pipe'
                if not os.path.isdir(eenos_pip_dir):
                    try:
                        os.makedirs(eenos_pip_dir)
                    except:
                        pass
                for domain in mailpips:
                    pffile=eenos_pip_dir+'/'+domain
                    try:
                        log("[ EMAIl ] Generating email pips of %s" %domain)
                        tools.writeyaml(pffile,mailpips[domain],log)
                    except Exception as e:
                        log(str(e))
                        pass
                # pprint(mailpips)
        # End of email alias,pips,forwarders
        if bool(accounts):
            maildir=workdir+'/homedir/mail'
            if not os.path.isdir(maildir):
                try:
                    os.makedirs(maildir)
                except Exception as e:
                    log(str(e))
                    pass
            for domain in accounts:
                for name in accounts[domain]:
                    source=backup+'/imap/'+domain+'/foo/Maildir'
                    target=maildir+'/'+domain+'/'+name
                    email=name+'@'+domain
                    if not os.path.isdir(target):
                        try:
                            os.makedirs(target)
                        except Exception as e:
                            log(str(e))
                            pass
                    if os.path.isdir(source):
                        cmd='/usr/bin/rsync --safe-links -avp '+source+'/ '+target+'/'
                        log("[ EMAIL ] Coping emails from %s" %email)
                        commands.live(cmd,log)


def collect_ssl_crts(backup,workdir,log):
    ssldir=workdir+'/userdata/ssl'
    crtfiles=glob.glob(backup+'/backup/*/domain.cert')
    ssls={}
    if bool(crtfiles):
        for crt in crtfiles:
            domain=str(crt).split("/")[-2]
            keyfile=str(crt).replace("domain.cert","domain.key")
            if os.path.isfile(keyfile):
                ssls[domain]={}
                ssls[domain]['crt']=crt
                ssls[domain]['key']=keyfile
    if bool(ssls):
        log("Collecting SSL certificates and keys")
        for domain in ssls:            
            crt_file=ssldir+'/'+domain+'-crt.pem'
            key_file=ssldir+'/'+domain+'-key.pem'
            try:
                log("[ %s ] Creating Eenos SSL certificate " %domain)
                shutil.copy(ssls[domain]['crt'],crt_file)
                security.safe_chmod(crt_file,0o644)
                log("[ %s ] Creating Eenos SSL Private Key " %domain)
                shutil.copy(ssls[domain]['key'],key_file)
                security.safe_chmod(key_file,0o640)
            except Exception as e:
                log(str(e))
                pass
    else:
        log("No SSL certificates")

def collect_domain_keys(backup,workdir,log):
    enosdkims=workdir+'/domain_keys'
    if not os.path.isdir(enosdkims):
        try:
            os.makedirs(enosdkims)
        except Exception as e:
            log(str(e))
            pass
    if not os.path.isdir(enosdkims+'/private'):
        try:
            os.makedirs(enosdkims+'/private')
            security.safe_chmod(enosdkims+'/private',0o750)
        except Exception as e:
            log(str(e))
            pass
    if not os.path.isdir(enosdkims+'/public'):
        try:
            os.makedirs(enosdkims+'/public')
        except Exception as e:
            log(str(e))
            pass
    pkfile=glob.glob(backup+'/backup/*/email/dkim.public.key')
    dkims={}
    if bool(pkfile):
        for pufile in pkfile:
            domain=str(pufile).split("/")[-3]
            prfile=str(pufile).replace("dkim.public.key","dkim.private.key")
            if os.path.isfile(prfile):
                dkims[domain]={}
                dkims[domain]['pub']=pufile
                dkims[domain]['pri']=prfile
    if bool(dkims):       
        for domain in dkims:
            pub_key=enosdkims+'/public/'+domain
            pri_key=enosdkims+'/private/'+domain
            try:
                log("[ DKIM ] Copying  public key of %s " %domain)
                shutil.copy(dkims[domain]['pub'],pub_key)
                security.safe_chmod(pri_key,0o644)
                log("[ DKIM ] Copying  private key of %s " %domain)
                shutil.copy(dkims[domain]['pri'],pri_key)
                security.safe_chmod(pri_key,0o640)
            except Exception as e:
                log(str(e))
                pass
    else:
        log("No Domain Keys available")    
        # pprint(dkims)

def collect_mysql_dbs(backup,workdir,log):
    eenos_sqldir=workdir+'/mysql/dbs'
    mfiles=glob.glob(backup+'/backup/*.sql')
    databases=dict()
    if bool(mfiles):
        log("Collecting MySQL Databases")
        for df in mfiles:
            sqlname=df.split('/')[-1]
            dbname=sqlname.split(".")[0]
            target=eenos_sqldir+'/'+sqlname
            try:
                log("[ %s ] Copying MySQL database dump - %s" %(dbname,sqlname))
                shutil.copy(df,target)
                databases[dbname]={}
                databases[dbname]['host']='localhost'
                databases[dbname]['user']={}
            except Exception as e:
                log(str(e))
                pass
    if bool(databases):
        ddata=workdir+'/userdata/mysql/databases'
        try:
            log("Creating Eenos MySQL database list")
            tools.writeyaml(ddata,databases,log)
        except Exception as e:
            log(str(e))
            pass

def collect_mysql_users(backup,workdir,user,userdata,hostname,log):    
    ddata=workdir+'/userdata/mysql/databases'
    if os.path.isfile(ddata):
        databases=tools.readyaml(ddata,log)
    else:
        databases=dict()
    dbprefix=user+'_'
    mysqlusers=dict()
    dbprivileges=dict()
    unsupported_passwords={}
    usersql={}
    if bool(databases):
        for db in databases:           
            dbconf=backup+'/backup/'+db+'.conf'
            if os.path.isfile(dbconf) and os.stat(dbconf).st_size > 1:
                # print(dbconf)
                dbcdata=tools.readplain(dbconf,log)                
                for item in dbcdata:
                    encitem=urllib.parse.unquote(item)                   
                    if str(encitem).startswith(dbprefix):                        
                        pra=str(encitem).split("&")                         
                        dbuser=pra[0].split("=")[0] 
                        if not dbuser in usersql:
                            usersql[dbuser]=dict()                            
                            usersql[dbuser]['querry']=list() 
                        password='NOPASSWORD'
                        for it in pra:
                            if it.startswith('passwd='):
                                password=it.split("=",1)[-1]
                        if not str(password).startswith('*'):                           
                            if not dbuser in unsupported_passwords:
                                log("[ !! Warning ] un-supported mysql user password for mysql user = %s = , creating a random password" %dbuser )
                                password=tools.getid()
                                unsupported_passwords[dbuser]=dict()
                                unsupported_passwords[dbuser]['info']='This mysql user was using unsupported password. Please use the following password.'
                                unsupported_passwords[dbuser]['password']=password
                            else:
                                password=unsupported_passwords[dbuser]['password']
                        password=tools.generate_mysql_pass(password)
                        dbprivileges[dbuser]={}
                        dbprivileges[dbuser]['password']=password 
                        databases[db]['user'][dbuser]=dict()
                        databases[db]['user'][dbuser]['grant']='ALL'
                        mysqlusers[dbuser]=dict()
                        mysqlusers[dbuser]['access_host']='localhost'
                        mysqlusers[dbuser]['host']='localhost'
                        usersql[dbuser]['password']=password                        
                        usersql[dbuser]['querry'].append(conf.mysql_user_create(mysql_user=dbuser,password=password,host='localhost'))
                        usersql[dbuser]['querry'].append(conf.mysql_user_grant_usage(mysql_user=dbuser,host='localhost'))
                        usersql[dbuser]['querry'].append(conf.mysql_user_grant_all(mysql_user=dbuser,db=db,host='localhost'))                                           
    if bool(usersql):        
        for dbuser in usersql:           
            if bool(usersql[dbuser]['querry']):
                cquery=usersql[dbuser]['querry']                
                nquery=list(set(cquery))
                nquery.sort()
                usersql[dbuser]['querry']=nquery
    
    if bool(usersql):
        user_query_dir=workdir+'/mysql/users'
        try:
            log("Updating Eenos MySQL database ownerships")
            tools.writeyaml(ddata,databases,log)
        except Exception as e:
            log(str(e))
            pass
        ddatauser=workdir+'/userdata/mysql/users'
        try:
            log("Updating Eenos MySQL database users list")
            tools.writeyaml(ddatauser,mysqlusers,log)
        except Exception as e:
            log(str(e))
            pass
        for sqluser in usersql:           
            querry=''
            if bool(usersql[sqluser]['querry']):
                for q in usersql[sqluser]['querry']:
                    querry+=q+'\n'
            usql=user_query_dir+'/'+sqluser+'.sql'
            try:
                log("[ %s ] Creating Eenos MySQL User create query file %s.sql " %(sqluser,sqluser))
                with open(usql,'w') as ur:
                    ur.write(querry)                    
            except Exception as e:
                log(str(e))
                pass
            if bool(unsupported_passwords):
                usfile=workdir+'/userdata/.new_mysql_users_password_delete_it_after_resetting_password.txt'
                try:
                   tools.writeyaml(usfile,unsupported_passwords,log)
                   security.safe_chmod(usfile,0o600)
                except Exception as e:
                    log(str(e))
                    pass
            # print(querry)
    else:
        log("No MySQL database users availabel")

def collect_mysql_dbs_users(backup,workdir,user,userdata,hostname,log):
    log("Collecting Directadmin MySQL Databases and MySQL users")
    emy_user_dir=workdir+'/mysql/users'
    emy_sql_dir=workdir+'/mysql/dbs'
    try:
        if not os.path.isdir(emy_user_dir):
            log("Creating MySQL user directory")
            os.makedirs(emy_user_dir)
        if not os.path.isdir(emy_sql_dir):
            log("Creating MySQL DB directory")
            os.makedirs(emy_sql_dir)
    except Exception as e:
        log(str(e))
        pass
    collect_mysql_dbs(backup,workdir,log)
    collect_mysql_users(backup,workdir,user,userdata,hostname,log)

def collect_homedir(backup,workdir,user,data,userdata,log):
    if bool(data['skiphomedir']):
        log("Skipping Home Directory")
    else:
        target=workdir+'/homedir'
        if not os.path.isdir(target):
            try:
                os.makedirs(target)
            except:
                pass
        source=None
        if os.path.isfile(backup+'/backup/home.tar.zst'):
            cmd='/usr/bin/zstd -d -f '+backup+'/backup/home.tar.zst'
            commands.live(cmd,log)
            if os.path.isfile(backup+'/backup/home.tar'):
                source=backup+'/backup/home.tar'
        if os.path.isfile(backup+'/backup/home.tar'):
            source=backup+'/backup/home.tar'
        if os.path.isfile(backup+'/backup/home.tar.gz'):
                source=backup+'/backup/home.tar.gz'
        if source and os.path.isdir(target):
            if tarfile.is_tarfile(source):
                try:
                    if str(source).endswith('.tar'):
                        cmd = 'tar -xvf '+source+' -C '+target
                    else:
                        cmd = 'tar -xvzf '+source+' -C '+target
                    log("Copying Homedir ..... please wait...")
                    # log(cmd)
                    commands.live(cmd,log)                    
                except Exception as e:
                    log(str(e))
                    pass
                tmaildir=target+'/Maildir'
                emaildir=target+'/mail'
                if not os.path.isdir(emaildir):
                    try:
                        os.makedirs(emaildir)
                    except:
                        pass
                if os.path.isdir(tmaildir):
                    try:
                        log("Copying default user emails ..")
                        cmd='/usr/bin/rsync --safe-links -avp '+tmaildir+'/ '+emaildir+'/'                        
                        commands.live(cmd,log) 
                        shutil.rmtree(tmaildir)
                    except Exception as e:
                        log(str(e))
                        pass  
            sdoms=backup+'/domains'
            if os.path.isdir(sdoms):
                cmd='/usr/bin/rsync --safe-links -avp '+sdoms+' '+target+'/'
                # log(cmd)
                commands.live(cmd,log) 
                domain=userdata['domain']
                maindata=target+'/domains/'+domain+'/public_html'
                if os.path.isdir(maindata):
                    log("Copying domain files ..")
                    cmd='/usr/bin/rsync --safe-links -avp '+maindata+' '+target+'/'
                    # log(cmd)
                    commands.live(cmd,log) 
                    try:
                        shutil.rmtree(maindata)
                        os.makedirs(maindata)
                    except Exception as e:
                        log(str(e))
                        pass
                else:
                    try:
                        os.makedirs(target+'/public_html')
                    except Exception as e:
                        log(str(e))
                        pass
                try:
                    if not os.path.islink(target+'/www'):
                        os.symlink('public_html',target+'/www' )
                except Exception as e:
                    log(str(e))
                    pass
        else:
            log("Unable to find homedir data skipping home")
        log("Homedir copy finished.")     

def collect_suspended(backup,workdir,user,log):
    esuspend=workdir+"/suspended"
    ucfile=backup+'/backup/user.conf'
    udata=tools.readconfig(ucfile,log)    
    if 'suspended' in udata and udata['suspended']=='yes':
        log("This account is suspened account, so making it suspened")
        log("Adding suspened  details")
        reason='Suspended from Diectadmin'
        try:
            if not os.path.isdir(esuspend):
                os.makedirs(esuspend)            
            esfile=esuspend+'/'+user
            sdata={}
            sdata['reason']=reason
            sdata['user']=user
            now=datetime.now()    
            times=now.strftime("%a, %d-%b-%Y %I:%M:%S, " )
            sdata['suspendtime']=str(times)   
            tools.writeyaml(esfile,sdata,log)
        except Exception as e:
            log(str(e))
            pass
   
def clean_tmp_bkp(backup,log):
    log("Cleaning temporary files .. ")
    try:
        shutil.rmtree(backup)
    except Exception as e:
        log(str(e))
        pass

def start(user,backup,data,log,live=True):
    workdir=data['myworkdir']+'/eenosbkp-'+user
    isworkdir=setup_work_dir(workdir,log)
    result=None
    if isworkdir:
        log('Workdir generated successfully  %s ' %workdir)
        try:
            edata={}
            edata['user']=user
            security.safe_chmod(workdir,0o700)
            get_shadow(backup,workdir,log)
            get_user_config(backup,workdir,user,data,log)
            eucf=workdir+'/userconfig/'+user
            userdata=tools.readyaml(eucf,log)
            if bool(userdata):
                havereseller=is_reseller(user,backup,log)
                if havereseller and not bool(data['skipresellerconfig']):
                    edata['reseller']='yes'
                    log("Collecting reseller settings .. ")
                    get_reseller_conf(userdata,backup,workdir,user,log)
                    get_reseller_packages(backup,workdir,user,log)
                    get_reseller_features(workdir,log)
                else:
                    edata['reseller']='no'
                hostname=collect_hostname()
                create_userdata_dirs(workdir,log)
                collect_domains(userdata,backup,workdir,user,log)
                colloect_ftp_accounts(backup,workdir,userdata,log)
                collect_cron_jobs(backup,workdir,user,log)
                collect_mail_filter_forwardor_pip(backup,workdir,data,userdata,log)
                collect_ssl_crts(backup,workdir,log)
                collect_domain_keys(backup,workdir,log)
                if bool(data['skipmysql']):
                    log("Skipping MySQL Databases and Users")
                else:
                    collect_mysql_dbs_users(backup,workdir,user,userdata,hostname,log) 
                # if bool(data['skippgsql']):
                #     log("Skipping PostgreSQL databases and users")
                # else:
                #     collect_pgql_dbs_users(backup,workdir,user,log) 
                edata['host']=hostname
                edata['backuptime']= str(datetime.now())
                edata['homedir']=userdata['homedir']
                edata['disk']=str(userdata['homedir']).rsplit('/',1)[0]
                tools.writeyaml(workdir+'/.eenos',edata,log) 
                collect_homedir(backup,workdir,user,data,userdata,log)
                collect_suspended(backup,workdir,user,log)
                # Remove temporary backup dir
                clean_tmp_bkp(backup,log)
                result=workdir
            else:
                log("Eenos user configuration generation failed")
                log("Directadmin to Eenos conversion failed")
        except Exception as e:
            log(str(e))
            result=None
            pass
    else:
        log('Unable to create workdir , Directadmin conversion failed for account : %s' %user)
        result=None
    # pprint(data)
    return result
       
